@@ -1,5 +1,5 @@
 // Aseprite Document IO Library
-// Copyright (c) 2018-2023 Igara Studio S.A.
+// Copyright (c) 2018-2025 Igara Studio S.A.
 // Copyright (c) 2001-2018 David Capello
 //
 // This file is released under the terms of the MIT license.
@@ -75,6 +75,9 @@ bool AsepriteDecoder::decode()
   sprite->setGridBounds(
     gfx::Rect(header.grid_x, header.grid_y, header.grid_width, header.grid_height));
 
+  sprite->useLayerUuids((header.flags & ASE_FILE_FLAG_LAYER_WITH_UUID) ==
+                        ASE_FILE_FLAG_LAYER_WITH_UUID);
+
   // Prepare variables for layer chunks
   doc::Layer* last_layer = sprite->root();
   doc::WithUserData* last_object_with_user_data = sprite.get();
@@ -577,46 +580,53 @@ doc::Layer* AsepriteDecoder::readLayerChunk(AsepriteHeader* header,
       break;
   }
 
-  if (layer) {
-    if (layer->isImage() &&
-        // Only transparent layers can have blend mode and opacity
-        !(flags & int(doc::LayerFlags::Background))) {
-      static_cast<doc::LayerImage*>(layer)->setBlendMode((doc::BlendMode)blendmode);
-      if (header->flags & ASE_FILE_FLAG_LAYER_WITH_OPACITY)
-        static_cast<doc::LayerImage*>(layer)->setOpacity(opacity);
-    }
+  if (!layer)
+    return nullptr;
 
-    // flags
-    layer->setFlags(
-      static_cast<doc::LayerFlags>(flags & static_cast<int>(doc::LayerFlags::PersistentFlagsMask)));
+  // Read UUID if usage is enabled
+  if (sprite->useLayerUuids())
+    layer->setUuid(readUuid());
 
-    // name
-    layer->setName(name.c_str());
+  const bool composeGroups = (header->flags & ASE_FILE_FLAG_COMPOSITE_GROUPS);
 
-    // Child level
-    if (child_level == *current_level)
-      (*previous_layer)->parent()->addLayer(layer);
-    else if (child_level > *current_level)
-      static_cast<doc::LayerGroup*>(*previous_layer)->addLayer(layer);
-    else if (child_level < *current_level) {
-      doc::LayerGroup* parent = (*previous_layer)->parent();
-      ASSERT(parent);
-      if (parent) {
-        int levels = (*current_level - child_level);
-        while (levels--) {
-          ASSERT(parent->parent());
-          if (!parent->parent())
-            break;
-          parent = parent->parent();
-        }
-        parent->addLayer(layer);
-      }
-    }
-
-    *previous_layer = layer;
-    *current_level = child_level;
+  if ((layer->isImage() || (layer->isGroup() && composeGroups)) &&
+      // Only transparent layers can have blend mode and opacity
+      !(flags & int(doc::LayerFlags::Background))) {
+    layer->setBlendMode((doc::BlendMode)blendmode);
+    if (header->flags & ASE_FILE_FLAG_LAYER_WITH_OPACITY)
+      layer->setOpacity(opacity);
   }
 
+  // flags
+  layer->setFlags(
+    static_cast<doc::LayerFlags>(flags & static_cast<int>(doc::LayerFlags::PersistentFlagsMask)));
+
+  // name
+  layer->setName(name.c_str());
+
+  // Child level
+  if (child_level == *current_level)
+    (*previous_layer)->parent()->addLayer(layer);
+  else if (child_level > *current_level)
+    static_cast<doc::LayerGroup*>(*previous_layer)->addLayer(layer);
+  else if (child_level < *current_level) {
+    doc::LayerGroup* parent = (*previous_layer)->parent();
+    ASSERT(parent);
+    if (parent) {
+      int levels = (*current_level - child_level);
+      while (levels--) {
+        ASSERT(parent->parent());
+        if (!parent->parent())
+          break;
+        parent = parent->parent();
+      }
+      parent->addLayer(layer);
+    }
+  }
+
+  *previous_layer = layer;
+  *current_level = child_level;
+
   return layer;
 }
 
@@ -824,6 +834,9 @@ doc::Cel* AsepriteDecoder::readCelChunk(doc::Sprite* sprite,
   int zIndex = ((int16_t)read16());
   readPadding(5);
 
+  if (cel_type == ASE_FILE_RAW_CEL || cel_type == ASE_FILE_COMPRESSED_CEL)
+    m_celType = cel_type;
+
   doc::Layer* layer = nullptr;
   if (layer_index >= 0 && layer_index < doc::layer_t(m_allLayers.size()))
     layer = m_allLayers[layer_index];
@@ -1428,12 +1441,7 @@ const doc::UserData::Variant AsepriteDecoder::readPropertyValue(uint16_t type)
       return value;
     }
     case USER_DATA_PROPERTY_TYPE_UUID: {
-      base::Uuid value;
-      uint8_t* bytes = value.bytes();
-      for (int i = 0; i < 16; ++i) {
-        bytes[i] = read8();
-      }
-      return value;
+      return readUuid();
     }
     default: {
       throw base::Exception(
@@ -1469,4 +1477,14 @@ void AsepriteDecoder::readTilesData(doc::Tileset* tileset, const AsepriteExterna
   }
 }
 
+base::Uuid AsepriteDecoder::readUuid()
+{
+  base::Uuid value;
+  uint8_t* bytes = value.bytes();
+  for (int i = 0; i < 16; ++i) {
+    bytes[i] = read8();
+  }
+  return value;
+}
+
 } // namespace dio
